#!/bin/bash

# set -x

# A lot checks because
# 1. its too easy to assign wrong values
# 2. number and semantics of values to be assigned to mysqltest_background.sh have changed too often
# 3. finding such bugs in the complicated MTR/testscript/mysqltest_background.sh mechanics is
#    unfortunately difficult. Sorry for that
if [ $# -ne 4 ]
then
    echo "ERROR: mysqltest_background.sh must be called with values for"
    echo "       MY_START_TS  == When the mysqltest runner should start to send the critical SQL "
    echo "       MY_END_TS    == When the mysqltest runner should exit"
    echo "       MY_WORKDIR   == Directory where the script for mysqltest is located"
    echo "       MY_NUMBER    == Number of the mysqltest runner (This is not related to --parallel)"
    echo "                       $0 <MY_START_TS> <MY_END_TS> <MY_WORKDIR> <MY_NUMBER>"
    echo "       The call was -->$0 $*<--"
    exit 8
fi
export MY_START_TS="$1"
export MY_END_TS="$2"
export MY_WORKDIR="$3"
export MY_NUMBER="$4"

MY_VAL=$(($MY_START_TS + 1 - 1));
if [ "$MY_START_TS" != "$MY_VAL" ]
then
    echo "ERROR: <MY_START_TS> == '$MY_START_TS' does not seem to be numeric."
    echo "                       $0 <MY_START_TS> <MY_END_TS> <MY_WORKDIR> <MY_NUMBER>"
    echo "       The call was -->$0 $*<--"
    exit 8
fi

MY_VAL=$(($MY_END_TS + 1 - 1));
if [ "$MY_END_TS" != "$MY_VAL" ]
then
    echo "ERROR: <MY_END_TS> == '$MY_END_TS' does not seem to be numeric."
    echo "                       $0 <MY_START_TS> <MY_END_TS> <MY_WORKDIR> <MY_NUMBER>"
    echo "       The call was -->$0 $*<--"
    exit 8
fi

if [ $MY_END_TS -le $MY_START_TS ]
then
    echo "ERROR: <MY_END_TS> == $MY_END_TS <= $MY_START_TS == <MY_START_TS>"
    echo "       MY_START_TS  == When the mysqltest runner should start to send the critical SQL "
    echo "       MY_END_TS    == When the mysqltest runner should exit"
    echo "                       $0 <MY_START_TS> <MY_END_TS> <MY_WORKDIR> <MY_NUMBER>"
    echo "       The call was -->$0 $*<--"
    exit 8
fi

if [ ! -d $MY_WORKDIR ]
then
    echo "ERROR: <MY_WORKDIR> == '$MY_WORKDIR' does not exist."
    echo "       MY_WORKDIR   == Directory where the script for mysqltest is located"
    echo "                       $0 <MY_START_TS> <MY_END_TS> <MY_WORKDIR> <MY_NUMBER>"
    echo "       The call was -->$0 $*<--"
    exit 8
fi

MY_VAL=$(($MY_NUMBER + 1 - 1));
if [ "$MY_NUMBER" != "$MY_VAL" ] 
then
    echo "ERROR: <MY_NUMBER> == '$MY_NUMBER' does not seem to be numeric."
    echo "                       $0 <MY_START_TS> <MY_END_TS> <MY_WORKDIR> <MY_NUMBER>"
    echo "       The call was -->$0 $*<--"
    exit 8
fi
export TST_FILE="$MY_WORKDIR""/sql.""$MY_NUMBER"
if [ ! -f $TST_FILE ]
then
    echo "ERROR: The TST_FILE '$TST_FILE' does not exist or is not a plain file."
    echo "       The (main) MTR test has to create that file before calling $0"
    echo "       The extension of that file must equal <MY_NUMBER> == '$MY_NUMBER'."
    echo "                       $0 <MY_START_TS> <MY_END_TS> <MY_WORKDIR> <MY_NUMBER>"
    echo "       The call was -->$0 $*<--"
    exit 8
fi
export RUN_FILE="$MY_WORKDIR""/run"
if [ ! -f $RUN_FILE ]
then
    echo "ERROR: The RUN_FILE '$RUN_FILE' does not exist or is not a plain file."
    echo "       The (main) MTR test has to create that file before calling $0"
    exit 8
fi


SOCKET="$MASTER_MYSOCK"
export OUT_FILE="$MY_WORKDIR""/out.""$MY_NUMBER"
export ERR_FILE="$MY_WORKDIR""/err.""$MY_NUMBER"
export PRT_FILE="$MY_WORKDIR""/prt.""$MY_NUMBER"
rm -f $PRT_FILE

if [ "$MTR_BINDIR" == '' ]
then
   $MTR_BINDIR=`dirname`
fi

if [ ! -f "$TST_FILE" ]
then
   echo "ERROR: TST_FILE '$TST_FILE' does not exist." >> "$PRT_FILE"
   echo "Abort" >> "$PRT_FILE"
   exit 4
fi


run_mysqltest()
{
   MY_CURRENT_TS=`date +%s`
   echo "MY_CURRENT_TS : $MY_CURRENT_TS , MY_START_TS : $MY_START_TS , MY_END_TS : $MY_END_TS"
   echo "MY_NUMBER : $MY_NUMBER , TST_FILE : '$TST_FILE'"
   echo "Begin of content of '$TST_FILE' ----"
   cat "$TST_FILE"
   echo "End   of content of '$TST_FILE' ----"
   if [ $MY_CURRENT_TS -gt $MY_END_TS ]
   then
      echo "ERROR: MY_CURRENT_TS ($MY_CURRENT_TS) > MY_END_TS ($MY_END_TS) ==> no mysqltest run."
      echo "Abort"
   else
      while [ $MY_CURRENT_TS -lt $MY_START_TS ]
      do
#        echo Ping $MY_CURRENT_TS
         sleep 0.1
         MY_CURRENT_TS=`date +%s`
      done

#     echo Peng $MY_CURRENT_TS

      while [ $MY_CURRENT_TS -lt $MY_END_TS ]
      do
         if [ ! -f "$RUN_FILE" ]
         then
            echo "The file '$RUN_FILE' does no more exist. Giving up."
            break
         fi

         echo "Start executing '$TST_FILE' at TS: $MY_CURRENT_TS"
         # OUT_FILE=/dev/null ; ERR_FILE=/dev/null;
         "$MTR_BINDIR"/client/mysqltest --user=root --database=test --socket="$SOCKET" --record \
            -x "$TST_FILE" > "$OUT_FILE" 2>"$ERR_FILE"
         RC=$?
         MY_CURRENT_TS=`date +%s`
         sync
         echo "End   executing '$TST_FILE' at TS: $MY_CURRENT_TS and status was $RC"
         echo "Begin of       output ----"
         cat $OUT_FILE
         echo "End   of       output ----"
         echo "Begin of error output ----"
         cat $ERR_FILE
         echo "End   of error output ----"
         rm -f "$OUT_FILE" "$ERR_FILE"
         sync
         if [ $RC -gt 0 ]
         then
            if [ ! -f "$RUN_FILE" ]
            then
               echo "The file '$RUN_FILE' does no more exist."
               echo "This means some concurrent session has observed a problem and deleted that file."
               echo "Assuming that this is the reason for my exit status > 0."
               break
            else
               echo "ERROR: My exit status > 0 means I have hit some not expected problem."
               echo "Removing the file '$RUN_FILE' and giving up."
               echo "Giving up."
               rm -rf "$RUN_FILE"
               sync
               # sleep 10
               exit $RC
            fi
         fi
         sleep 1.0
         MY_CURRENT_TS=`date +%s`
      done
   fi

   echo ENDE $MY_CURRENT_TS
   sync
}

run_mysqltest >> "$PRT_FILE" 2>&1 &

